/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XConfigFile.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak (pHreak@x-treme-online.de)
//
// Erstelldatum:	17.08.2001
// letzte Aenderung:17.08.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Die Klasse XConfigFile dient zur Ermittlung von Parametern aus
//					einer Konfigurations-Datei.
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme (www.x-treme-online.de)
/////////////////////////////////////////////////////////////////////////////////////

// includes
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "XLogging.h"
#include "XConfigFile.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::XConfigFile
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor
/////////////////////////////////////////////////////////////////////////////////////

XConfigFile::XConfigFile(const char	*pcConfigFile)
{
	m_pcConfigFile = new char[strlen(pcConfigFile)+1];

	strcpy(m_pcConfigFile,pcConfigFile);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::~XConfigFile
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor
/////////////////////////////////////////////////////////////////////////////////////

XConfigFile::~XConfigFile()
{
	if (m_pcConfigFile)
	{
		delete [] m_pcConfigFile;
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::parseKeyValuePair
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Holt ein Schluessel/Wert-Paar aus der Datei.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XConfigFile::parseKeyValuePair(const char	*pcParam,
									   char			*pcValue,
									   size_t		sizeValue)
{
	m_file.open(m_pcConfigFile,ios::in|ios::nocreate);

	if (m_file.fail())
	{
		#if defined(XDEBUG) || defined(XCONFIGFILEDEBUG)
		XLogging::log("XConfigFile::parseKeyValuePair > (WARNUNG!!!) Datei \"");
		XLogging::log(m_pcConfigFile);
		XLogging::log("\" nicht gefunden\n");
		#endif
		
		return ERROR_FILE_NOT_FOUND;
	}

	bool	bIsWithinQuotationMark	= false;
	bool	bIsKeyFound				= false;
	bool	bIsGettingValue			= false;
	int		nMatchingChars			= 0;
	int		nPosition				= 0;
	int		nSlashCount				= 0;
	
	char	cLineToEval[4096];
	char	cTempLine[8192];
	char	cActualChar;

	HRESULT	hRes = ERROR_OBJECT_NOT_FOUND;

	while (!m_file.eof())
	{
		cTempLine[0]	= '\0';
		nPosition		= 0;
		nMatchingChars	= 0;
		nSlashCount		= 0;
		bIsKeyFound		= false;
		
		// Zeile einlesen
		m_file.getline(cLineToEval,sizeof(cLineToEval));
		cLineToEval[sizeof(cLineToEval)-1]	= '\0';

		// Zeichenweise parsen
		for (int nI=0;nI<(int) strlen(cLineToEval);++nI)
		{
			cTempLine[nPosition++]	= (cActualChar = cLineToEval[nI]);

			// NICHT innerhalb von ""
			if (bIsWithinQuotationMark==false)
			{
				if (bIsKeyFound)
				{
					if (cActualChar=='=')
					{
						bIsKeyFound		= false;
						bIsGettingValue	= true;
						nPosition		= 0;
						cTempLine[0]	= '\0';

						continue; // for (int nI=...
					}

					// Whitespace ueberlesen
					if ((cActualChar=='\t') ||
						(cActualChar=='\n') ||
						(cActualChar==' '))
					{
						continue; // for (int nI=...
					}

					bIsKeyFound	= false;
				} // if (bIsKeyFound)

				if (cActualChar==(pcParam[nMatchingChars]))
				{
					if (++nMatchingChars==(int) strlen(pcParam))
					{
						bIsKeyFound	= true;
					}

					continue; // for (int nI=...
				}

				nMatchingChars	= 0;

				if (cActualChar=='/')
				{
					if (++nSlashCount==2)
					{ // wir haben einen Kommentar erwischt
						// Position um 2 zuruecksetzen
						nPosition -= 2;
						break; // Parsing fuer diese Zeile abbrechen
					}
					
					continue; // for (int nI=...
				}

				nSlashCount		= 0;

			} // if (bIsWithinQuotationMark==false)

			// " gefunden
			if (cActualChar=='\"')
			{
				bIsWithinQuotationMark = !bIsWithinQuotationMark;
			}
		} // for (int nI...

        cTempLine[nPosition]	= '\0';

		if (bIsGettingValue==true)
		{
			int nFirstNonWhitespace = 0; // Ist das dann ein Blackblock?

			while ((cTempLine[nFirstNonWhitespace]=='\n') ||
				   (cTempLine[nFirstNonWhitespace]=='\t') ||
				   (cTempLine[nFirstNonWhitespace]=='\"') ||
				   (cTempLine[nFirstNonWhitespace]==' '))
			{
				nFirstNonWhitespace++;
			}

			while ((cTempLine[nPosition-1]=='\n') ||
				   (cTempLine[nPosition-1]=='\t') ||
				   (cTempLine[nPosition-1]=='\"') ||
				   (cTempLine[nPosition-1]==' '))
			{
				cTempLine[--nPosition]	= '\0';
			}

			strncpy(pcValue,cTempLine+nFirstNonWhitespace,sizeValue);

			hRes = S_OK;

			break; // while-Schleife
		}
	} // while (!m_file.eof())

	m_file.close();

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::getAsString
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Gibt einen Parameter-Wert als String zurueck.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XConfigFile::getAsString(const char	*pcParam,
								 char		*pcValue,
								 size_t		sizeValue,
								 const char	*pcDefault)
{
	HRESULT hRes = parseKeyValuePair(pcParam,pcValue,sizeValue);

	if (hRes!=S_OK)
	{
		strncpy(pcValue,pcDefault,sizeValue);

		#if defined(XDEBUG) || defined(XCONFIGFILEDEBUG)
		XLogging::log("XConfigFile::getAsString > (WARNUNG!!!) Parameter \"");
		XLogging::log(pcParam);
		XLogging::log("\" nicht gefunden, benutze Default-Wert: \"");
		XLogging::log(pcDefault);
		XLogging::log("\"\n");
		#endif
	}

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::getAsInt
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Gibt einen Parameter-Wert als int zurueck.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XConfigFile::getAsInt(const char	*pcParam,
							  int			&nValue,
							  int			nDefault)
{
	char	cValue[1024];
	
	HRESULT hRes = parseKeyValuePair(pcParam,cValue,sizeof(cValue));

	if (hRes!=S_OK)
	{
		nValue	= nDefault;

		#if defined(XDEBUG) || defined(XCONFIGFILEDEBUG)
		XLogging::log("XConfigFile::getAsInt > (WARNUNG!!!) Parameter \"");
		XLogging::log(pcParam);
		XLogging::log("\" nicht gefunden, benutze Default-Wert: ");
		XLogging::log(nDefault);
		XLogging::log("\n");
		#endif
	}
	else
	{
		nValue = atoi(cValue);
	}

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::getAsDouble
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Gibt einen Parameter-Wert als double zurueck.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XConfigFile::getAsDouble(const char	*pcParam,
								 double		&dValue,
								 double		dDefault)
{
	char	cValue[1024];
	
	HRESULT hRes = parseKeyValuePair(pcParam,cValue,sizeof(cValue));

	if (hRes!=S_OK)
	{
		dValue	= dDefault;

		#if defined(XDEBUG) || defined(XCONFIGFILEDEBUG)
		XLogging::log("XConfigFile::getAsDouble > (WARNUNG!!!) Parameter \"");
		XLogging::log(pcParam);
		XLogging::log("\" nicht gefunden, benutze Default-Wert: ");
		XLogging::log(dDefault);
		XLogging::log("\n");
		#endif
	}
	else
	{
		dValue = atof(cValue);
	}

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XConfigFile::getAsBool
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Gibt einen Parameter-Wert als bool zurueck.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XConfigFile::getAsBool(const char	*pcParam,
							   bool			&bValue,
							   bool			bDefault)
{
	char	cValue[1024];
	
	HRESULT hRes = parseKeyValuePair(pcParam,cValue,sizeof(cValue));

	if (hRes!=S_OK)
	{
		bValue	= bDefault;

		#if defined(XDEBUG) || defined(XCONFIGFILEDEBUG)
		XLogging::log("XConfigFile::getAsBool > (WARNUNG!!!) Parameter \"");
		XLogging::log(pcParam);
		XLogging::log("\" nicht gefunden, benutze Default-Wert: ");
		XLogging::log(bDefault?"true":"false");
		XLogging::log("\n");
		#endif
	}
	else
	{
		bValue = (stricmp(cValue,"true")==0);
	}

	return hRes;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XConfigFile.cpp
/////////////////////////////////////////////////////////////////////////////////////