/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XDisplay.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	23.08.2000
// letzte Aenderung:17.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Ein Objekt der Klasse XDisplay verkoerpert das Ausgabemedium. Es
//					verwaltet zum einen das DirectX-Objekt, zum anderen die Primary
//					und Backbuffer Surfaces. Da die Klasse von XSurface erbt, koennen
//					ihre Objekte ueberall dort uebergeben werden, wo eine XSurface
//					erwartet wird.
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

#define DIRECTDRAW_VERSION 0x0700 // Dann klappts auch mit dem "DirectX>7-SDK"-Nachbarn

// XGame-includes
#include "XDisplay.h"

DWORD	XDisplay::m_dwMaskRed	= 0;
DWORD	XDisplay::m_dwMaskGreen	= 0;
DWORD	XDisplay::m_dwMaskBlue	= 0;

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::XDisplay
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XDisplay, initialisiert alle Datenmember der
//					Klasse.
/////////////////////////////////////////////////////////////////////////////////////

XDisplay::XDisplay(void)
{
	resetData();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt alle Datenmember der Klasse zurueck
/////////////////////////////////////////////////////////////////////////////////////

void XDisplay::resetData(void)
{
	m_dwMaskBlue		= 0;
	m_dwMaskGreen		= 0;
	m_dwMaskRed			= 0;

	m_lpDirect3D7		= NULL;
	m_lpDirect3DDevice7	= NULL;
	m_lpDirectDraw7		= NULL;
	m_lpPrimarySurface	= NULL;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::~XDisplay
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor von XDisplay, ruft XDisplay::uninitialize auf
/////////////////////////////////////////////////////////////////////////////////////

XDisplay::~XDisplay(void)
{
	// Die Pruefung, ob das Objekt initialisiert ist, findet schon hier statt damit
	// keine WARNUNG ausgegeben wird, wenn ein Objekt erzeugt, aber nie verwendet
	// wird. Siehe hierzu auch XDisplay::uninitialize.
	
	if (m_bIsInitialized==true)
	{
		uninitialize();
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt ein DirectDraw7-Objekt, setzt den Cooperative Level,
//					schaltet in den gewuenschten Grafikmodus, legt eine Primary
//					Surface mit attached Backbuffer an und erzeugt einen Clipper, der
//					alle Blit-Operationen abschneidet, die ueber die Backbuffer
//					Surface hinausgehen wuerden.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XDisplay::initialize(DWORD	dwWidth,
							 DWORD	dwHeight,
							 DWORD	dwBitsPerPixel,
							 HWND	hWnd,
							 DWORD	dwFlags,
							 bool	b3DNeeded)
{
	D3DVIEWPORT7	D3DViewport7;
	HRESULT			hDDResult;
	DDSURFACEDESC2	DDSurfaceDesc2;
	DDSCAPS2		DDSCaps;
		
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	if (m_bIsInitialized==true)
	{
		#if defined(XDEBUG) || defined(XDisplayDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Display bereits initialisiert\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}
	#endif
	
	// DirectDraw7-Objekt anlegen
	hDDResult = DirectDrawCreateEx(NULL,(void **)&m_lpDirectDraw7,IID_IDirectDraw7,NULL);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte DirectDraw7-Objekt nicht erzeugen\n");
		#endif
		
		m_lpDirectDraw7 = NULL;
		
		return hDDResult;
	}

	m_bIsInitialized	= true;

	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > DirectDraw7-Objekt erzeugt\n");
	#endif	
	
	// Cooperative Level setzen
	hDDResult = m_lpDirectDraw7->SetCooperativeLevel(hWnd,dwFlags);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Cooperative Level nicht setzen\n");
		#endif
		uninitialize();
		
		return hDDResult;
	}
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > Cooperative Level gesetzt\n");
	#endif	
	
	// Display Modus setzen
	hDDResult = m_lpDirectDraw7->SetDisplayMode(dwWidth,dwHeight,dwBitsPerPixel,0,0);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Display Modus nicht setzen\n");
		#endif
		uninitialize();
		
		return hDDResult;
	}
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > Display Modus gesetzt\n");
	#endif	
	
	// Primary Surface erzeugen
	ZeroMemory(&DDSurfaceDesc2,sizeof(DDSURFACEDESC2));

	DDSurfaceDesc2.dwSize			= sizeof(DDSurfaceDesc2);
    DDSurfaceDesc2.dwFlags			= DDSD_CAPS|DDSD_BACKBUFFERCOUNT;
	DDSurfaceDesc2.ddsCaps.dwCaps	= DDSCAPS_PRIMARYSURFACE|DDSCAPS_FLIP|DDSCAPS_COMPLEX;
    if (b3DNeeded==true)
	{
		DDSurfaceDesc2.ddsCaps.dwCaps	|= DDSCAPS_3DDEVICE;
	}
	DDSurfaceDesc2.dwBackBufferCount= 1;
    
	hDDResult = m_lpDirectDraw7->CreateSurface(&DDSurfaceDesc2,&m_lpPrimarySurface,NULL);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Primary Surface nicht erzeugen\n");
		#endif
		uninitialize();
		
		return hDDResult;
	}
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > Primary Surface erzeugt\n");
	#endif	

	// Adresse der Backbuffer Surface holen
	ZeroMemory(&DDSCaps,sizeof(DDSCaps));
	DDSCaps.dwCaps = DDSCAPS_BACKBUFFER;
	hDDResult = m_lpPrimarySurface->GetAttachedSurface(&DDSCaps,&m_lpSurface);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Adresse der Backbuffer Surface nicht ermitteln\n");
		#endif
		uninitialize();
		
		return hDDResult;
	}
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > Adresse der Backbuffer Surface ermittelt\n");
	#endif	
	
	// Datenmember der Backbuffer Surface initialisieren
	m_lWidth	= dwWidth;
	m_lHeight	= dwHeight;

	// ClippingRect auf gesamte Surface
	RECT	rectClipper;
	SetRect(&rectClipper,0,0,dwWidth,dwHeight);

	setClippingRect(&rectClipper);

	// Pixelformat holen
	ZeroMemory(&DDSurfaceDesc2,sizeof(DDSURFACEDESC2));
	DDSurfaceDesc2.dwSize	= sizeof(DDSurfaceDesc2);

	hDDResult = m_lpDirectDraw7->GetDisplayMode(&DDSurfaceDesc2);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Display Modus nicht ermitteln\n");
		#endif
		uninitialize();

		return hDDResult;
	}
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	XLogging::log("XDisplay::initialize > Display Modus ermittelt\n");
	#endif	

	m_dwMaskRed		= DDSurfaceDesc2.ddpfPixelFormat.dwRBitMask;
	m_dwMaskGreen	= DDSurfaceDesc2.ddpfPixelFormat.dwGBitMask;
	m_dwMaskBlue	= DDSurfaceDesc2.ddpfPixelFormat.dwBBitMask;

	if (b3DNeeded==true)
	{
		if (m_lpDirectDraw7->QueryInterface(IID_IDirect3D7,(LPVOID *) &m_lpDirect3D7)!=S_OK)
		{
			#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
			XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Direct3D7-Interface nicht ermitteln\n");
			#endif

			return DDERR_INVALIDPARAMS;
		}
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > Direct3D7-Interface ermittelt\n");
		#endif	

		if (hDDResult=m_lpDirect3D7->CreateDevice(IID_IDirect3DTnLHalDevice,m_lpSurface,&m_lpDirect3DDevice7)!=D3D_OK)
		{
			#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
			XLogging::log("XDisplay::initialize > Konnte TnL-Device nicht erzeugen, versuche HAL-Device\n");
			#endif
			if (hDDResult=m_lpDirect3D7->CreateDevice(IID_IDirect3DHALDevice,m_lpSurface,&m_lpDirect3DDevice7)!=D3D_OK)
			{
				#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
				XLogging::log("XDisplay::initialize > Konnte HAL-Device nicht erzeugen, versuche MMX-Device\n");
				#endif
				if (hDDResult=m_lpDirect3D7->CreateDevice(IID_IDirect3DMMXDevice,m_lpSurface,&m_lpDirect3DDevice7)!=D3D_OK)
				{
					#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
					XLogging::log("XDisplay::initialize > Konnte MMX-Device nicht erzeugen, versuche RGB-Device\n");
					#endif
					if (hDDResult=m_lpDirect3D7->CreateDevice(IID_IDirect3DRGBDevice,m_lpSurface,&m_lpDirect3DDevice7)!=D3D_OK)
					{
						#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
						XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte kein Direct3D-Device erzeugen\n");
						#endif

						return hDDResult;
					}
					#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
					else
					{
						XLogging::log("XDisplay::initialize > RGB-Device erzeugt\n");
					}
					#endif
				}
				#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
				else
				{
					XLogging::log("XDisplay::initialize > MMX-Device erzeugt\n");
				}
				#endif
			}
			#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
			else
			{
				XLogging::log("XDisplay::initialize > HAL-Device erzeugt\n");
			}
			#endif
		}
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		else
		{
			XLogging::log("XDisplay::initialize > TnL-Device erzeugt\n");
		}
		#endif

		D3DViewport7.dwX		= 0;
		D3DViewport7.dwY		= 0; 
		D3DViewport7.dwWidth	= dwWidth;
		D3DViewport7.dwHeight	= dwHeight; 
		D3DViewport7.dvMinZ		= 0.0f;
		D3DViewport7.dvMaxZ		= 1.0f;
		hDDResult	= m_lpDirect3DDevice7->SetViewport(&D3DViewport7);

		if (hDDResult!=DD_OK)
		{
			#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
			XLogging::log("XDisplay::initialize > (FEHLER!!!) Konnte Viewport nicht setzen\n");
			#endif

			uninitialize();

			return hDDResult;
		}
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::initialize > Viewport gesetzt\n");
		#endif
	}

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt alle in XDisplay::initialize erstellten Objekte -sofern
//					vorhanden- in dieser Reihenfolge frei:
//					1. Direct3DDevice7
//					2. Direct3D7-Objekt	
//					3. Primary Surface (damit stirbt auch die attached Backbuffer
//					   Surface)
//					4. DirectDraw7-Objekt
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XDisplay::uninitialize(void)
{
	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::uninitialize > (WARNUNG!!!) Display nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}

	// Direct3DDevice7 freigeben
	if (m_lpDirect3DDevice7)
	{
		m_lpDirect3DDevice7->Release();
		m_lpDirect3DDevice7	= NULL;

		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::uninitialize > Direct3DDevice7 freigegeben\n");
		#endif
	}

	// Direct3D7-Objekt freigeben
	if (m_lpDirect3D7)
	{
		m_lpDirect3D7->Release();
		m_lpDirect3D7	= NULL;

		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::uninitialize > Direct3D7-Objekt freigegeben\n");
		#endif
	}

	// Primary Surface freigeben
	if (m_lpPrimarySurface)
	{
		m_lpPrimarySurface->Release();
		m_lpPrimarySurface	= NULL;
		// Durch Freigabe der Primary Surface wird automatisch auch die Backbuffer
		// Surface zerstoert. Da XDisplay von XSurface erbt, wuerde der Aufruf des
		// Destruktors von XSurface erneut versuchen, die nicht mehr vorhandene
		// Backbuffer Surface freizugeben. Um die erneute Freigabe zu vermeiden,
		// wird m_bIsInitialized auf false gesetzt und alle Datenmember der XSurface
		// zurueckgesetzt. Vergleiche hierzu auch den Destruktor von XSurface.
		m_bIsInitialized	= false;
		XSurface::resetData();
				
		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::uninitialize > Primary Surface und Backbuffer Surface freigegeben\n");
		#endif
	}
	
	// DirectDraw7-Objekt freigeben
	if (m_lpDirectDraw7)
	{
		m_lpDirectDraw7->Release();
		m_lpDirectDraw7	= NULL;
		m_bIsInitialized	= false;

		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		XLogging::log("XDisplay::uninitialize > DirectDraw7-Objekt freigegeben\n");
		#endif
	}

	// Member ruecksetzen
	resetData();

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::flip
/////////////////////////////////////////////////////////////////////////////////////
// Info:			verursacht einen Austausch von Backbuffer Surface und Primary
//					Surface.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XDisplay::flip(DWORD	dwFlags)
{
	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	if (!m_lpPrimarySurface)
	{
		XLogging::log("XDisplay::flip > (FEHLER!!!) Primary Surface existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif
	
	return m_lpPrimarySurface->Flip(NULL,dwFlags);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::setClippingRect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt das ClippingRect der Display
/////////////////////////////////////////////////////////////////////////////////////

void XDisplay::setClippingRect(LPRECT	lpClippingRect)
{
	D3DVIEWPORT7	D3DViewport7;
	HRESULT			hDDResult;

	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	if ((lpClippingRect->right<0) ||
		(lpClippingRect->right>m_lWidth) ||
		(lpClippingRect->left<0) ||
		(lpClippingRect->left>m_lWidth) ||
		(lpClippingRect->top<0) ||
		(lpClippingRect->top>m_lHeight) ||
		(lpClippingRect->bottom<0) ||
		(lpClippingRect->bottom>m_lHeight) ||
		(lpClippingRect->left>lpClippingRect->right) ||
		(lpClippingRect->top>lpClippingRect->bottom))
	{
		XLogging::log("XDisplay::setClippingRect > (FEHLER!!!) ClippingRect-Daten ungltig (");
		XLogging::log(lpClippingRect->left);
		XLogging::log(",");
		XLogging::log(lpClippingRect->top);
		XLogging::log("/");
		XLogging::log(lpClippingRect->right);
		XLogging::log(",");
		XLogging::log(lpClippingRect->bottom);
		XLogging::log(")\n");

		return;
	}
	#endif

	m_rectClippingRect	= *lpClippingRect;

	if (m_lpDirect3DDevice7)
	{ // Viewport anpassen
		D3DViewport7.dwX		= lpClippingRect->left;
		D3DViewport7.dwY		= lpClippingRect->top; 
		D3DViewport7.dwWidth	= lpClippingRect->right-lpClippingRect->left;
		D3DViewport7.dwHeight	= lpClippingRect->bottom-lpClippingRect->top; 
		D3DViewport7.dvMinZ		= 0.0f;
		D3DViewport7.dvMaxZ		= 1.0f;
		
		hDDResult	= m_lpDirect3DDevice7->SetViewport(&D3DViewport7);

		#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
		if (hDDResult!=DD_OK)
		{
			XLogging::log("XDisplay::setClippingRect > (FEHLER!!!) Konnte Viewport nicht setzen\n");

			return;
		}
		#endif
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::restore
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Restauriert die verlorene Primary Surface und alle implizit
//					erstellten Surfaces (Backbuffer)
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XDisplay::restore(void)
{
	HRESULT				hDDResult;

	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	if (m_lpPrimarySurface==NULL)
	{
		XLogging::log("XDisplay::restore > (FEHLER!!!) Primary Surface existiert nicht\n");
		
		return DDERR_INVALIDPARAMS;
	}
	#endif
	
	// Surface restoren,...
	hDDResult	= m_lpPrimarySurface->Restore();

	#if defined(XDEBUG) || defined(XDISPLAYDEBUG)
	if (hDDResult!=DD_OK)
	{
		XLogging::log("XDisplay::restore > (FEHLER!!!) restore fehlgeschlagen\n");
	}
	#endif

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XDisplay::convertRGBToColorModel
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konvertiert einen RGB-Wert in ein DWORD, das als ColorKey fuer
//					das aktuelle Color Model genutzt werden kann.
/////////////////////////////////////////////////////////////////////////////////////

DWORD XDisplay::convertRGBToColorModel(WORD	wRed,
									   WORD	wGreen,
									   WORD	wBlue)
{
	DWORD	dwRed;
	DWORD	dwGreen;
	DWORD	dwBlue;
	WORD	wBits;
	DWORD	dwMask;

	// Rot
	wBits=1;
	dwMask	= XDisplay::m_dwMaskRed;

	while (dwMask=dwMask>>1)
	{
		wBits++;
	}
	dwRed	= DWORD (wRed) << wBits;

	// Gruen
	wBits	= 1;
	dwMask	= XDisplay::m_dwMaskGreen;

	while (dwMask=dwMask>>1)
	{
		wBits++;
	}
	dwGreen	= DWORD (wGreen) << wBits;

	// Blau
	wBits	= 1;
	dwMask	= XDisplay::m_dwMaskBlue;

	while (dwMask=dwMask>>1)
	{
		wBits++;
	}
	dwBlue	= DWORD (wBlue) << wBits;

	return (((dwRed>>8)&XDisplay::m_dwMaskRed) |
			((dwGreen>>8)&XDisplay::m_dwMaskGreen) |
			((dwBlue>>8)&XDisplay::m_dwMaskBlue));
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XDisplay.cpp
/////////////////////////////////////////////////////////////////////////////////////