/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XPlay.h
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Baal
//
// Version			1.1
// Erstelldatum:	18.09.2000
// letzte nderung:	19.02.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
//
//
//
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000 by x-treme
/////////////////////////////////////////////////////////////////////////////////////


#define WIN32_LEAN_AND_MEAN		// Selten genutzte Teile aus Win-Header ausschliessen

#ifndef _XPLAY_H
#define _XPLAY_H


// Standard Includes
#include <windows.h>
#include <dplay.h>


// Eigene Includes
#include "XLogging.h"


#define X_MSG 0x0001


/////////////////////////////////////////////////////////////////////////////////
// Struktur XMessage

struct XMessage
{
	// Muss immer als Member vorhanden sein
	DWORD	m_dwType;
	
	// Diese Werte sind optional - sie knnen jenach Verwendung angepasst werden
	char	m_strMessage[25];
	int		m_nXPosition;
	int		m_nYPosition;
};



/////////////////////////////////////////////////////////////////////////////////
// Klasse XPlayer

class XPlayer
{
	friend class XPlayerManager;

	private:
		XPlayer();
		virtual ~XPlayer();
		
		LPSTR	m_lpShortPlayerName;
		LPSTR	m_lpLongPlayerName;
		LPDPID	m_lpPlayerId;
		DWORD	m_dwPlayerType;
		
		XPlayer	*m_pNext;
};

// Ende Klasse XPlayer
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XSession

class XSession
{
	friend class XSessionManager;

	private:
		XSession();
		virtual ~XSession();
		
		LPGUID		m_lpGuid;
		LPSTR		m_lpShortName;	
		
		XSession	*m_pNext;
};

// Ende Klasse XConnection
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XConnection

class XConnection
{
	friend class XConnectionManager;

	private:
		XConnection();
		virtual ~XConnection();
		
		LPVOID		m_lpConnection;
		LPSTR		m_lpShortName;
		
		XConnection *m_pNext;
};

// Ende Klasse XConnection
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XPlayerManager

class XPlayerManager
{
	friend class XPlay;

	private:
		XPlayerManager();

		HRESULT addPlayer(DPID pidID, DWORD dwPlayerType, LPCDPNAME lpName);

		LPSTR getFirstPlayerName();
		LPSTR getNextPlayerName();
		LPDPID getFirstPlayer();
		LPDPID getNextPlayer();
		LPSTR getPlayer(LPDPID playerId);
		
		HRESULT releasePlayers();

		int		m_nCount;
		XPlayer *m_pFirst;
		XPlayer *m_pCurrent;
};

// Ende Klasse XPlayerManager
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XSessionManager

class XSessionManager
{
	friend class XPlay;

	private:
		XSessionManager();
		
		HRESULT addSession(LPDPSESSIONDESC2 lpSessionDesc);

		LPSTR getFirstSessionName();
		LPSTR getNextSessionName();
		LPGUID getFirstSession();
		LPGUID getNextSession();
		LPSTR getSession(LPGUID lpSession);
		
		HRESULT releaseSessions();

		int			m_nCount;
		XSession	*m_pFirst;
		XSession	*m_pCurrent;
};

// Ende Klasse XSessionManager
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XConnectionManager

class XConnectionManager
{
	friend class XPlay;

	private:
		XConnectionManager();

		HRESULT addConnection(LPCGUID lpMyGuidSP, 
							LPVOID lpMyConnection, 
							DWORD dwMyConnectionSize, 
							LPCDPNAME lpMyName, 
							DWORD dwMyFlags, 
							LPVOID lpMyContext);
		
		LPSTR getFirstConnectionName();
		LPSTR getNextConnectionName();
		LPVOID getFirstConnection();
		LPVOID getNextConnection();
		LPVOID getConnection(LPSTR connectionName);

		HRESULT releaseConnections();

		int			m_nCount;
		XConnection	*m_pFirst;
		XConnection	*m_pCurrent;
};

// Ende Klasse XConnectionManager
/////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////
// Klasse XPlay

class XPlay
{
	private:
		static BOOL WINAPI enumConnectionCallback(LPCGUID lpguidSP, 
													LPVOID lpConnection, 
													DWORD dwConnectionSize, 
													LPCDPNAME lpName, 
													DWORD dwFlags, 
													LPVOID lpContext);
		
		static BOOL WINAPI enumSessionCallback(LPDPSESSIONDESC2 lpSessionDesc, 
												LPDWORD lpdwTimeOut, 
												DWORD dwFlags, 
												LPVOID lpContext);

		static BOOL WINAPI enumPlayerCallback(DPID pidID, 
												DWORD dwPlayerType, 
												LPCDPNAME lpName, 
												DWORD dwFlags, 
												LPVOID lpContext);

		
		static XConnectionManager	m_pConnectionManager;
		static XSessionManager		m_pSessionManager;
		static XPlayerManager		m_pPlayerManager;
		
		LPDIRECTPLAY4A				m_lpDirectPlay;

	protected:

	public:
		XPlay();
		virtual ~XPlay();
		
		HRESULT	initialize();
		
		HRESULT	enumConnections();
		LPSTR	getFirstConnectionName();
		LPSTR	getNextConnectionName();
		LPVOID	getFirstConnection();
		LPVOID	getNextConnection();
		LPVOID	getConnection(LPSTR connectionName);
		HRESULT	initializeConnection(LPVOID pOurConnection);
		HRESULT releaseConnections();

		HRESULT	enumSessions(GUID g_Guid = GUID_NULL);
		LPSTR	getFirstSessionName();
		LPSTR	getNextSessionName();
		LPGUID	getFirstSession();
		LPGUID	getNextSession();
		LPSTR	getSession(LPGUID lpSession);
		HRESULT createSession(LPSTR sessionName, GUID g_Guid = GUID_NULL);
		HRESULT joinSession(LPGUID sessionGuid, GUID g_Guid = GUID_NULL);
		HRESULT releaseSessions();
		
		HRESULT enumPlayers(LPGUID lpSessionGuid = NULL, DWORD dwPlayerType = DPENUMPLAYERS_ALL);
		LPSTR	getFirstPlayerName();
		LPSTR	getNextPlayerName();
		LPDPID	getFirstPlayer();
		LPDPID	getNextPlayer();
		LPSTR	getPlayer(LPDPID playerId);
		HRESULT createPlayer(HANDLE hPlayerEvent = NULL, LPSTR playerName = "Player", DWORD dwPlayerType = 0);
		HRESULT releasePlayers();

		HRESULT sendMessage(LPVOID lpData, DWORD dwDataSize, DPID dpFromId, DPID dpToId = DPID_ALLPLAYERS, DWORD dwFlags = 0);
		HRESULT receiveMessage(DPID lpFromId, DPID lpToId, DWORD dwFlags = 0);
		
		HRESULT	uninitialize();

		LPVOID	m_lpMessageBuffer;
		DWORD	m_dwBufferSize;
};

// Ende Klasse XPlay
/////////////////////////////////////////////////////////////////////////////////


#endif//_XPLAY_H


class	XPlayer;
class	XSession;
class	XConnection;
class	XPlayerManager;
class	XSessionManager;
class	XConnectionManager;
class	XPlay;


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XPlay.h
/////////////////////////////////////////////////////////////////////////////////////