/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSpriteManager.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	19.09.2000
// letzte nderung:	20.02.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
//
//
//
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

// XGame-includes
#include "XSpriteManager.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::XSpriteManagerListItem
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XSpriteManagerListItem. Initialisiert alle 
//					Datenmember
/////////////////////////////////////////////////////////////////////////////////////

XSpriteManagerListItem::XSpriteManagerListItem(XSprite*	pXSpriteToInsert)
{
	bProcessed	= true;
	
	pNext		=
	pPrevious	= NULL;

	pXSprite	= pXSpriteToInsert;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::XSpriteManager
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XSpriteManager. Initialisiert alle Datenmember
/////////////////////////////////////////////////////////////////////////////////////

XSpriteManager::XSpriteManager(void)
{
	pFirst	= NULL;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::~XSpriteManager
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor von XSpriteManager
/////////////////////////////////////////////////////////////////////////////////////

XSpriteManager::~XSpriteManager(void)
{
	removeAll();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::insertSprite
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Fuegt das uebergebene Sprite pXSprite in die Verwaltungsliste des
//					Spritemanagers ein.
/////////////////////////////////////////////////////////////////////////////////////

XSprite* XSpriteManager::insertSprite(XSprite* pXSprite)
{
	XSpriteManagerListItem*	pNewListItem;

	if ((pXSprite) && (pXSprite->isInitialized()==false))
	{
		#ifdef XDEBUG
		XLogging::log("XSpriteManager::insertSprite > (WARNUNG!!!) Einzufgendes Sprite nicht initialisiert\n");
		#endif

		return NULL;
	}
	
	pNewListItem	= new XSpriteManagerListItem(pXSprite);

	if (!pNewListItem)
	{
		#ifdef XDEBUG
		XLogging::log("XSpriteManager::insertSprite > (FEHLER!!!) Zuwenig Speicher fr XSpriteManagerListItem\n");
		#endif

		return NULL;
	}
	
	// Neues Element als erstes Element der Liste einfuegen
	if (pFirst)
	{
		pNewListItem->pNext	= pFirst;
		pFirst->pPrevious	= pNewListItem;
	}
	pFirst	= pNewListItem;
	
	#ifdef XDEBUG
	LPSTR	lpstrDebugString1="XSpriteManager::insertSprite > Sprite \"",
			lpstrDebugString2="\" eingefgt\n",
			lpstrDebugString;
	
	lpstrDebugString = new char[strlen(lpstrDebugString1)+
								strlen(pXSprite->m_lpstrFilename)+
								strlen(lpstrDebugString2)+1];

	strcpy(lpstrDebugString,lpstrDebugString1);
	strcat(lpstrDebugString,pXSprite->m_lpstrFilename);
	strcat(lpstrDebugString,lpstrDebugString2);

	XLogging::log(lpstrDebugString);
	
	delete lpstrDebugString;
	#endif
	
	return pXSprite;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::removeAll
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Entfernt alle Sprites aus der Verwaltungsliste des Spritemanagers
/////////////////////////////////////////////////////////////////////////////////////

long XSpriteManager::removeAll(void)
{
	XSpriteManagerListItem*	pListItemBuffer;
	long					lCount = 0;

	while (pFirst)
	{
		lCount++;
		pListItemBuffer = pFirst;
		pFirst			= pFirst->pNext;
		
		// Sprite rauskegeln ;-)
		delete pListItemBuffer;

		#ifdef XDEBUG
		XLogging::log("XSpriteManager::removeAll > Sprite entfernt\n");
		#endif
	}

	return lCount; // zurueckgeben, wieviele Sprites in der Kette waren
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::removeSprite
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Entfernt ein bestimmtes Sprites aus der Verwaltungsliste des 
//					Spritemanagers
/////////////////////////////////////////////////////////////////////////////////////

XSprite* XSpriteManager::removeSprite(XSprite*	pXSprite)
{
	XSpriteManagerListItem*	pListItemBuffer;
	XSpriteManagerListItem*	pActual	= pFirst;

	while ((pActual) && (pActual->pXSprite!=pXSprite))
	{
		pActual	= pActual->pNext;
	}

	if (!pActual) // Sprite nicht vorhanden
	{
		#ifdef XDEBUG
		XLogging::log("XSpriteManager::removeSprite > (WARNUNG!!!) zu entfernendes Sprite existiert nicht\n");
		#endif

		return NULL;
	}
	// sonst: Sprite aus der Liste ausschliessen
	pListItemBuffer = pActual->pNext;

	if (pActual->pPrevious) // wenn wir einen Vorgaenger haben...
	{
		// ...Vorgaenger auf Nachfolger von pActual zeigen lassen
		// pActual->pPrevious->pNext	= pActual->pNext;
		
		pActual->pPrevious->pNext	= pListItemBuffer; // sollte schneller sein
	}
	else
	{
		// sonst waren wir der Erste und der Nachfolger wird zum Ersten
		// pFirst	= pActual->pNext;

		pFirst	= pListItemBuffer; // sollte schneller sein
	}
	
	if (pActual->pNext) // wenn wir einen Nachfolger haben...
	{
		// ...Nachfolger auf Vorgaenger von pActual zeigen lassen
		pActual->pNext->pPrevious	= pActual->pPrevious;
	}
	
	// Sprite rauskegeln ;-)
	delete pActual;

	#ifdef XDEBUG
	XLogging::log("XSpriteManager::removeSprite > Sprite entfernt\n");
	#endif
	
	return pXSprite;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::animateAll
/////////////////////////////////////////////////////////////////////////////////////
// Info:			ruft animate fuer jedes Sprite aus der Verwaltungsliste des 
//					aus Spritemanagers
/////////////////////////////////////////////////////////////////////////////////////

void XSpriteManager::animateAll(void)
{
	XSpriteManagerListItem*	pListItemBuffer;
	XSpriteManagerListItem*	pActual	= pFirst;

	while (pActual)
	{
		pListItemBuffer = pActual->pNext;

		if (pActual->pXSprite->animate()==false)
		{
			// Sprite aus der Liste ausschliessen
			if (pActual->pPrevious) // wenn wir einen Vorgaenger haben...
			{
				// ...Vorgaenger auf Nachfolger von pActual zeigen lassen
				// pActual->pPrevious->pNext	= pActual->pNext;
				
				pActual->pPrevious->pNext	= pListItemBuffer; // sollte schneller sein
			}
			else
			{
				// sonst waren wir der Erste und der Nachfolger wird zum Ersten
				// pFirst	= pActual->pNext;

				pFirst	= pListItemBuffer; // sollte schneller sein
			}
			
			if (pActual->pNext) // wenn wir einen Nachfolger haben...
			{
				// ...Nachfolger auf Vorgaenger von pActual zeigen lassen
				pActual->pNext->pPrevious	= pActual->pPrevious;
			}
			
			// Sprite rauskegeln ;-)
			delete pActual;
		}
		
		// zum naechsten Element hangeln
		pActual	= pListItemBuffer;
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSpriteManager::blitAll
/////////////////////////////////////////////////////////////////////////////////////
// Info:			ruft blit fuer jedes Sprite aus der Verwaltungsliste des 
//					aus Spritemanagers
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSpriteManager::blitAll(XSurface*	lpDestinationXSurface,
								DWORD		dwFlags)
{
	XSpriteManagerListItem*	pActual	= pFirst;
	XSpriteManagerListItem*	pMaxZ	= pFirst;
	long					lMaxZ	= -1000000;
	long					lCount	= 0;
	HRESULT					hRes	= DD_OK;

	while (pActual)
	{
		// alle Sprites als noch nicht abgearbeitet markieren
		pActual->bProcessed	= false;
		pActual				= pActual->pNext;
		
		// nebenbei zaehlen, wieviele Objekte wir haben
		lCount++;
	}

	while (lCount!=0)
	{
		// wir fangen vorne an zu suchen
		pActual	= pFirst;

		while (pActual)
		{
			if ((pActual->bProcessed==false) && // Sprite noch nicht abgearbeitet *UND* ...
				(pActual->pXSprite->m_lZ>lMaxZ)) // ...das Sprite liegt "tiefer"
			{
				// aktuelles Sprite als "tiefstes" vermerken...
				pMaxZ	= pActual;
				// ...und dessen Z-Koordinate als Maximum speichern
				lMaxZ	= pActual->pXSprite->m_lZ;
			}
			pActual	= pActual->pNext; // weiterhangeln
		}
		// sind wir hier angekommen, haben wir das "tiefste" nicht abgearbeitete Sprite
		// in der Variablen pMaxZ 
		
		// also: blitten
		if ((hRes = pMaxZ->pXSprite->blit(lpDestinationXSurface,dwFlags))!=DD_OK)
			return hRes;
		// als abgearbeitet markieren
		pMaxZ->bProcessed	= true;
		// und lMaxZ wieder auf einen *SEHR* kleinen Wert setzen
		lMaxZ				= -1000000;

		lCount--; // wir haben wieder ein Sprite abgearbeitet
	}

	return hRes;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSpriteManager.cpp
/////////////////////////////////////////////////////////////////////////////////////