/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSurface.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	25.08.2000
// letzte Aenderung:17.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Die Klasse XSurface verwaltet eine DirectDraw7-Surface.
//					Die Klasse stellt Funktionen zum Laden von Grafiken aus .BMP-
//					Dateien zur Verfuegung. Zusaetzliche Methoden erlauben blit-
//					Operationen in andere XSurfaces
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

#define DIRECTDRAW_VERSION 0x0700 // Dann klappts auch mit dem "DirectX>7-SDK"-Nachbarn

// XGame-includes
#include "XDisplay.h"
#include "XSurface.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::XSurface
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XSurface, initialisiert alle Datenmember der
//					Klasse.
/////////////////////////////////////////////////////////////////////////////////////

XSurface::XSurface(void)
{
	resetData();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt alle Datenmember der Klasse zurueck
/////////////////////////////////////////////////////////////////////////////////////

void XSurface::resetData(void)
{
	m_lRefCount			= 0;
	
	m_bIsInitialized	= false;
	m_bIsLocked			= false;

	m_lpSurface			= NULL;
	m_lpRectLocked		= NULL;

	m_lWidth			=
	m_lHeight			= 0;

	m_lpstrFilename		= NULL;

	m_ftu				= 0.0f;
	m_ftv				= 0.0f;

	SetRect(&m_rectClippingRect,0,0,0,0);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::~XSurface
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor von XSurface, ruft XSurface::uninitialize auf
/////////////////////////////////////////////////////////////////////////////////////

XSurface::~XSurface(void)
{
	// Die Pruefung, ob das Objekt initialisiert ist, findet schon hier statt damit
	// keine WARNUNG ausgegeben wird, wenn ein Objekt erzeugt, aber nie verwendet
	// wird. Siehe hierzu auch XSurface::uninitialize.
	
	if (m_bIsInitialized==true)
	{
		uninitialize();
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt eine DirectDraw Surface und laedt eine Grafikdatei in die
//					Surface
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::initialize(XDisplay*		pXDisplay,
							 LPSTR			lpstrFilename,
							 LPDDCOLORKEY	lpDDColorKey,
							 DWORD			dwFlags)
{
	long	lWidth;
	long	lHeight;
		
	if ((getBitmapSize(lpstrFilename,&lWidth,&lHeight)!=DD_OK) ||
		(initialize(pXDisplay,lWidth,lHeight,lpDDColorKey,dwFlags)!=DD_OK))
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) Initialisierung fehlgeschlagen\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}
	
	// Dateinamen kopieren
	m_lpstrFilename = new char[strlen(lpstrFilename)+1];
	if (!m_lpstrFilename)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) Zuwenig Speicher fr Filename\n");
		#endif

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}
	strcpy(m_lpstrFilename,lpstrFilename);

	// Bitmap laden
	if (loadBitmapIntoSurface(m_lpstrFilename)!=DD_OK)
	{
		uninitialize();
	
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) Initialisierung fehlgeschlagen\n");
		#endif

		uninitialize();

		return DDERR_INVALIDOBJECT;
	}

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	XLogging::log("XSurface::initialize > Surface \"");
	XLogging::log(m_lpstrFilename);
	XLogging::log("\" erzeugt\n");
	#endif

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt eine DirectDraw Surface uebergebener Groesse
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::initialize(XDisplay*		pXDisplay,
							 long			lWidth,
							 long			lHeight,
							 LPDDCOLORKEY	lpDDColorKey,
							 DWORD			dwFlags)
{
	HRESULT				hDDResult;
	DDSURFACEDESC2		DDSurfaceDesc2;
	D3DDEVICEDESC7		DD3DDeviceDesc7;
	RECT				rectClipper;
		
	if (m_bIsInitialized==true)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) Surface bereits initialisiert\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}
	if ((!pXDisplay) || (!pXDisplay->m_lpDirectDraw7))
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) DirectDraw7-Objekt existiert nicht\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}
	
	// Breite und Hoehe setzen
	m_lWidth	= lWidth;
	m_lHeight	= lHeight;
	
	// Surface-Description setzen
	ZeroMemory(&DDSurfaceDesc2,sizeof(DDSURFACEDESC2));

	DDSurfaceDesc2.dwSize			= sizeof(DDSurfaceDesc2);
	DDSurfaceDesc2.dwFlags			= DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH;
	DDSurfaceDesc2.ddsCaps.dwCaps	= dwFlags;
    DDSurfaceDesc2.dwWidth			= m_lWidth;
    DDSurfaceDesc2.dwHeight			= m_lHeight;

	// ColorKey setzen
	if (lpDDColorKey)
	{
		DDSurfaceDesc2.ddckCKSrcBlt.dwColorSpaceLowValue	= lpDDColorKey->dwColorSpaceLowValue;
		DDSurfaceDesc2.ddckCKSrcBlt.dwColorSpaceHighValue	= lpDDColorKey->dwColorSpaceHighValue;
		DDSurfaceDesc2.dwFlags							   |= DDSD_CKSRCBLT;
    }

	// Surface als Textur zu verwenden? (Alphablitting)
	if ((dwFlags&DDSCAPS_TEXTURE)==DDSCAPS_TEXTURE)
	{
		if (pXDisplay->m_lpDirect3DDevice7==NULL)
		{
			#if defined(XDEBUG) || defined(XSURFACEDEBUG)
			XLogging::log("XSurface::initialize > (FEHLER!!!) Direct3D-Device existiert nicht\n");
			#endif

			resetData();
			
			return DDERR_INVALIDOBJECT;
		}
		
		// Caps des Direct3D-Devices holen
		pXDisplay->m_lpDirect3DDevice7->GetCaps(&DD3DDeviceDesc7);

		// Sind wir hardwarebeschleunigt?
		if ((DD3DDeviceDesc7.deviceGUID==IID_IDirect3DHALDevice) ||
			(DD3DDeviceDesc7.deviceGUID==IID_IDirect3DTnLHalDevice))
		{ // JA! Dann soll sich der Treiber um die Verwaltung der Textur kuemmern
			DDSurfaceDesc2.ddsCaps.dwCaps2	= DDSCAPS2_TEXTUREMANAGE;
		}
		else
		{ // NEIN! Textur in den Arbeitsspeicher, damit die CPU schnelleren Zugriff hat
			DDSurfaceDesc2.ddsCaps.dwCaps	|= DDSCAPS_SYSTEMMEMORY;
		}
		
		if (DD3DDeviceDesc7.dpcTriCaps.dwTextureCaps&D3DPTEXTURECAPS_POW2)
		{
			// Surface-Groesse auf 2er-Potenz setzen
			for (DDSurfaceDesc2.dwWidth=1;
				 (unsigned long) m_lWidth>DDSurfaceDesc2.dwWidth;
				 DDSurfaceDesc2.dwWidth<<=1);
			for (DDSurfaceDesc2.dwHeight=1;
				 (unsigned long) m_lHeight>DDSurfaceDesc2.dwHeight;
				 DDSurfaceDesc2.dwHeight<<=1);
		}
		
		// tu und tv bestimmen
		m_ftu		= (float) (m_lWidth+1) / (float) DDSurfaceDesc2.dwWidth;
		m_ftv		= (float) (m_lHeight+1) / (float) DDSurfaceDesc2.dwHeight;

		if(DD3DDeviceDesc7.dpcTriCaps.dwTextureCaps&D3DPTEXTURECAPS_SQUAREONLY)
		{
			// Surface-Ausmasse quadratisch setzen
			if(DDSurfaceDesc2.dwWidth>DDSurfaceDesc2.dwHeight)
			{
				DDSurfaceDesc2.dwHeight	= DDSurfaceDesc2.dwWidth;
			}
			else
			{
				DDSurfaceDesc2.dwWidth	= DDSurfaceDesc2.dwHeight;
			}
		}
	}

	// Surface erzeugen
	hDDResult = pXDisplay->m_lpDirectDraw7->CreateSurface(&DDSurfaceDesc2,&m_lpSurface,NULL);
	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::initialize > (FEHLER!!!) Konnte Surface nicht erzeugen\n");
		#endif
		
		resetData();
		
		return hDDResult;
	}
	
	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	XLogging::log("XSurface::initialize > Surface erzeugt (");
	XLogging::log(m_lWidth);
	XLogging::log("x");
	XLogging::log(m_lHeight);
	XLogging::log(")\n");
	#endif

	// Reference Zaehler erhoehen
	addRef();

	// Clipper auf die gesamte Surface setzen
	SetRect(&rectClipper,0,0,m_lWidth,m_lHeight);
	setClippingRect(&rectClipper);

	// jetzt ist die Surface initialisiert
	m_bIsInitialized	= true;

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Surface frei
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::uninitialize(void)
{
	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::uninitialize > (WARNUNG!!!) Surface nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}
	
	if (!(--m_lRefCount))
	{
		m_lpSurface->Release();
		m_lpSurface	= NULL;
				
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		if (m_lpstrFilename==NULL)
		{
			XLogging::log("XSurface::uninitialize > Surface freigegeben (");
			XLogging::log(m_lWidth);
			XLogging::log("x");
			XLogging::log(m_lHeight);
			XLogging::log(")\n");
		}
		else
		{
			XLogging::log("XSurface::uninitialize > Surface freigegeben \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\"\n");
		}
		#endif
	}
	else
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		if (m_lpstrFilename==NULL)
		{
			XLogging::log("XSurface::uninitialize > Referenz-Zhler dekrementiert\n");
		}
		else
		{
			XLogging::log("XSurface::uninitialize > Referenz-Zhler von Surface \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\" dekrementiert\n");
		}
		#endif

		return DD_OK;
	}

	// Speicher fuer Filename freigeben
	if (m_lpstrFilename)
	{
		delete [] m_lpstrFilename;
		m_lpstrFilename	= NULL;
	}
	
	// Member ruecksetzen
	resetData();

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::restore
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Restauriert eine verlorene Surface. Laedt auch die Bitmap-Daten
//					neu.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::restore(void)
{
	HRESULT				hDDResult;

	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::restore > (FEHLER!!!) Surface nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}

	// Surface restoren,...
	hDDResult	= m_lpSurface->Restore();

	if (hDDResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		if (m_lpstrFilename==NULL)
		{
			XLogging::log("XSurface::restore > (FEHLER!!!) restore fehlgeschlagen (");
			XLogging::log(m_lWidth);
			XLogging::log("x");
			XLogging::log(m_lHeight);
			XLogging::log(")\n");
		}
		else
		{
			XLogging::log("XSurface::restore > (FEHLER!!!) restore fehlgeschlagen \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\"\n");
		}
		#endif

		return hDDResult;
	}

	// ...neu laden...
	if (m_lpstrFilename!=NULL)
	{
		if (loadBitmapIntoSurface(m_lpstrFilename)!=DD_OK)
		{
			#if defined(XDEBUG) || defined(XSURFACEDEBUG)
			XLogging::log("XSurface::restore > (FEHLER!!!) restore fehlgeschlagen \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\"\n");
			#endif

			return DDERR_INVALIDOBJECT;
		}
		
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		if (m_lpstrFilename==NULL)
		{
			XLogging::log("XSurface::restore > Surface restored (");
			XLogging::log(m_lWidth);
			XLogging::log("x");
			XLogging::log(m_lHeight);
			XLogging::log(")\n");
		}
		else
		{
			XLogging::log("XSurface::restore > Surface restored \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\"\n");
		}
		#endif
	}

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::blit
/////////////////////////////////////////////////////////////////////////////////////
// Info:			kopiert das Rechteck lpRectSource der rufenden Surface an die 
//					Position lX,lY der Surface lpDestinationSurface 
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::blit(XSurface*	lpDestinationXSurface,
					   long			lX,
					   long			lY,
					   LPRECT		lpRectSource,
					   DWORD		dwFlags)
{
	HRESULT	hResult;
	RECT	rectSrc;
	RECT	rectDst;
	RECT	rectDstClipper;

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XSurface::blit > (FEHLER!!!) Surface nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!lpDestinationXSurface)
	{
		XLogging::log("XSurface::blit > (FEHLER!!!) Ziel-Surface existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif
	
	if (lpRectSource)
	{ // Rect uebergeben?
		SetRect(&rectDst,lX,lY,
				lX+(lpRectSource->right-lpRectSource->left),
				lY+(lpRectSource->bottom-lpRectSource->top));
		SetRect(&rectSrc,lpRectSource->left,lpRectSource->top,
				lpRectSource->right,lpRectSource->bottom);
	}
	else
	{ // wenn nicht, ganze Surface nehmen
		SetRect(&rectDst,lX,lY,lX+m_lWidth,lY+m_lHeight);
		SetRect(&rectSrc,0,0,m_lWidth,m_lHeight);
	}
	rectDstClipper	= lpDestinationXSurface->getClippingRect();

	if ((rectDst.left>=rectDstClipper.right) ||
		(rectDst.top>=rectDstClipper.bottom) ||
		(rectDst.right<=rectDstClipper.left) ||
		(rectDst.bottom<=rectDstClipper.top))
	{ // kein Blit auszufuehren, da Ziel ausserhalb Clipper liegt
		return DD_OK;
	}
	
	// Clipping...
	// ... rechts
	if (rectDst.right>rectDstClipper.right)
	{
		rectSrc.right  -= rectDst.right-rectDstClipper.right;
		rectDst.right	= rectDstClipper.right;
	}
	// ...links
	if (rectDst.left<rectDstClipper.left)
	{
		rectSrc.left   += rectDstClipper.left-rectDst.left;
		rectDst.left	= rectDstClipper.left;
	}
	// ...oben
	if (rectDst.top<rectDstClipper.top)
	{
		rectSrc.top+= rectDstClipper.top-rectDst.top;
		rectDst.top	= rectDstClipper.top;
	}
	// ...unten
	if (rectDst.bottom>rectDstClipper.bottom)
	{
		rectSrc.bottom -= rectDst.bottom-rectDstClipper.bottom;
		rectDst.bottom	= rectDstClipper.bottom;
	}

	hResult = lpDestinationXSurface->m_lpSurface->Blt(&rectDst,
													  m_lpSurface,
													  &rectSrc,
													  dwFlags,
													  NULL);

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (hResult!=DD_OK)
	{
		XLogging::log("XSurface::blit > (FEHLER!!!) Blit fehlgeschlagen\n");
	}
	#endif

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::blitAlpha
/////////////////////////////////////////////////////////////////////////////////////
// Info:			kopiert das Rechteck lpRectSource der rufenden Surface an die 
//					Position lX,lY der Surface lpDestinationSurface 
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::blitAlpha(XDisplay*		lpXDisplay,
							long			lX,
							long			lY,
							unsigned char	ucAlpha,
							DWORD			dwFlags)
{
	HRESULT		hResult;
	D3DTLVERTEX	square[4];
	float		fX1,
				fY1,
				fX2,
				fY2;

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XSurface::blitAlpha > (FEHLER!!!) Surface nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!lpXDisplay)
	{
		XLogging::log("XSurface::blitAlpha > (FEHLER!!!) Display existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif
	
	fX1	= (float) lX;
	fY1	= (float) lY;
	fX2	= (float) lX+m_lWidth;
	fY2	= (float) lY+m_lHeight;
	
	square[0]	= D3DTLVERTEX(D3DVECTOR(fX1,fY1,0.0f),1.0f,RGBA_MAKE(255,255,255,ucAlpha),0,0.0f,0.0f);
	square[1]	= D3DTLVERTEX(D3DVECTOR(fX2,fY1,0.0f),1.0f,RGBA_MAKE(255,255,255,ucAlpha),0,m_ftu,0.0f);
	square[2]	= D3DTLVERTEX(D3DVECTOR(fX1,fY2,0.0f),1.0f,RGBA_MAKE(255,255,255,ucAlpha),0,0.0f,m_ftv);
	square[3]	= D3DTLVERTEX(D3DVECTOR(fX2,fY2,0.0f),1.0f,RGBA_MAKE(255,255,255,ucAlpha),0,m_ftu,m_ftv);

	lpXDisplay->m_lpDirect3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,true);
	lpXDisplay->m_lpDirect3DDevice7->SetRenderState(D3DRENDERSTATE_SRCBLEND,D3DBLEND_SRCALPHA);
	lpXDisplay->m_lpDirect3DDevice7->SetRenderState(D3DRENDERSTATE_DESTBLEND,D3DBLEND_INVSRCALPHA);
	lpXDisplay->m_lpDirect3DDevice7->SetRenderState(D3DRENDERSTATE_COLORKEYENABLE,true);
	lpXDisplay->m_lpDirect3DDevice7->SetTextureStageState(0,D3DTSS_ALPHAOP,D3DTOP_MODULATE);
	lpXDisplay->m_lpDirect3DDevice7->SetTexture(0,m_lpSurface);

	hResult	= lpXDisplay->m_lpDirect3DDevice7->BeginScene();
	
	if (hResult==D3D_OK)
	{
		lpXDisplay->m_lpDirect3DDevice7->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_TLVERTEX,square,4,0);
		lpXDisplay->m_lpDirect3DDevice7->SetTexture(0,NULL); 
		lpXDisplay->m_lpDirect3DDevice7->EndScene();
	}

	lpXDisplay->m_lpDirect3DDevice7->SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE,false);

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (hResult!=DD_OK)
	{
		XLogging::log("XSurface::blitAlpha > (FEHLER!!!) Blit fehlgeschlagen\n");
	}
	#endif

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::lock
/////////////////////////////////////////////////////////////////////////////////////
// Info:			sperrt die Surface vollstaendig (lpRectToLock=NULL) oder
//					innerhalb der Region lpRectToLock und gibt in lplpSurface einen
//					Pointer auf die Surface-Daten zurueck
//
// ACHTUNG !!!		Lock sperrt scheinbar alle OutputDebugString-Ausgaben
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::lock(LPVOID*	lplpSurface,
					   LPRECT	lpRectToLock,
					   DWORD	dwFlags)
{
	HRESULT			hResDD;
	DDSURFACEDESC2	SurfaceDesc2;

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XSurface::lock > (FEHLER!!!) Surface nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	if (m_bIsLocked==true)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::lock > (WARNUNG!!!) Surface bereits gesperrt\n");
		#endif

		return DD_OK;
	}
	
	// SurfaceDesc2 vorbereiten
	ZeroMemory(&SurfaceDesc2,sizeof(SurfaceDesc2));
	SurfaceDesc2.dwSize = sizeof(SurfaceDesc2);
	
	// Surface sperren
	hResDD = m_lpSurface->Lock(lpRectToLock,&SurfaceDesc2,dwFlags,NULL);

	if (hResDD!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::lock > (FEHLER!!!) Konnte Surface nicht sperren\n");
		#endif
		
		*lplpSurface = NULL;

		return hResDD;
	}
	
	m_lpRectLocked	= lpRectToLock;
	m_bIsLocked		= true;

	*lplpSurface	= SurfaceDesc2.lpSurface;
	
	return hResDD;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::unlock
/////////////////////////////////////////////////////////////////////////////////////
// Info:			entsperrt die Surface
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::unlock(void)
{
	HRESULT	hResDD;

	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XSurface::unlock > (FEHLER!!!) Surface nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	if (m_bIsLocked==false)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::unlock > (WARNUNG!!!) Surface war nicht gesperrt\n");
		#endif

		return DD_OK;
	}

	hResDD		= m_lpSurface->Unlock(m_lpRectLocked);
	m_bIsLocked	= (hResDD!=DD_OK);
	
	return hResDD;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::getWidth
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Breite der Surface zurueck
/////////////////////////////////////////////////////////////////////////////////////

long XSurface::getWidth(void)
{
	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("Surface::getWidth > (WARNUNG!!!) Surface nicht initialisiert\n");
	}
	#endif
	
	return m_lWidth;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::getHeight
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Hoehe der Surface zurueck
/////////////////////////////////////////////////////////////////////////////////////

long XSurface::getHeight(void)
{
	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("Surface::getHeight > (WARNUNG!!!) Surface nicht initialisiert\n");
	}
	#endif
	
	return m_lHeight;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::addRef
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erhoeht den Reference Zaehler der Surface
/////////////////////////////////////////////////////////////////////////////////////

long XSurface::addRef(void)
{
	return (++m_lRefCount);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::getRefCount
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt den Reference Zaehler der Surface zurueck
/////////////////////////////////////////////////////////////////////////////////////

long XSurface::getRefCount(void)
{
	return m_lRefCount;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::loadBitmapIntoSurface
/////////////////////////////////////////////////////////////////////////////////////
// Info:			laedt eine Bitmap in die Surface
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::loadBitmapIntoSurface(LPSTR	lpstrFilename)
{
	HBITMAP	hBitmap	= 0;
	HRESULT	hResult	= DD_OK;
	HDC		hDCSource,
			hDCDestination;
	
	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::loadBitmapIntoSurface > (FEHLER!!!) Surface nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}

	// Grafik laden
	hBitmap = (HBITMAP) LoadImage(0,lpstrFilename,IMAGE_BITMAP,0,0,LR_LOADFROMFILE|LR_CREATEDIBSECTION);

	if (hBitmap==0)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::loadBitmapIntoSurface > (FEHLER!!!) Konnte Bitmap \"");
		XLogging::log(lpstrFilename);
		XLogging::log("\" nicht laden\n");
		#endif

		return DDERR_NOTFOUND;
	}

	hDCSource = CreateCompatibleDC(NULL);
	if (!hDCSource)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::loadBitmapIntoSurface > (FEHLER!!!) Konnte CompatibleDC nicht erzeugen\n");
		#endif
		
		DeleteObject(hBitmap);
		
		return DDERR_INVALIDOBJECT;
	}
	SelectObject(hDCSource,hBitmap);
	
	hResult = m_lpSurface->GetDC(&hDCDestination);
	if (hResult!=DD_OK)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::loadBitmapIntoSurface > (FEHLER!!!) Konnte DC der Surface nicht ermitteln\n");
		#endif
		
		DeleteDC(hDCSource);
		DeleteObject(hBitmap);
		
		return DDERR_INVALIDOBJECT;
	}
	if (StretchBlt(hDCDestination,0,0,m_lWidth,m_lHeight,hDCSource,0,0,m_lWidth,m_lHeight,SRCCOPY)==0)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::loadBitmapIntoSurface > (FEHLER!!!) Konnte Grafik nicht in Surface blitten\n");
		#endif

		// DC freigeben
		m_lpSurface->ReleaseDC(hDCDestination);
		DeleteDC(hDCSource);
		DeleteObject(hBitmap);
		
		return DDERR_INVALIDOBJECT;
	}

	// DC freigeben
	m_lpSurface->ReleaseDC(hDCDestination);
	DeleteDC(hDCSource);
	DeleteObject(hBitmap);

	return hResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::setClippingRect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt das ClippingRect der Surface
/////////////////////////////////////////////////////////////////////////////////////

void XSurface::setClippingRect(LPRECT	lpClippingRect)
{
	#if defined(XDEBUG) || defined(XSURFACEDEBUG)
	if ((lpClippingRect->right<0) ||
		(lpClippingRect->right>m_lWidth) ||
		(lpClippingRect->left<0) ||
		(lpClippingRect->left>m_lWidth) ||
		(lpClippingRect->top<0) ||
		(lpClippingRect->top>m_lHeight) ||
		(lpClippingRect->bottom<0) ||
		(lpClippingRect->bottom>m_lHeight) ||
		(lpClippingRect->left>lpClippingRect->right) ||
		(lpClippingRect->top>lpClippingRect->bottom))
	{
		XLogging::log("XSurface::setClippingRect > (FEHLER!!!) ClippingRect-Daten ungltig (");
		XLogging::log(lpClippingRect->left);
		XLogging::log(",");
		XLogging::log(lpClippingRect->top);
		XLogging::log("/");
		XLogging::log(lpClippingRect->right);
		XLogging::log(",");
		XLogging::log(lpClippingRect->bottom);
		XLogging::log(")\n");

		return;
	}
	#endif

	m_rectClippingRect	= *lpClippingRect;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::getClippingRect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt das ClippingRect der Surface zurueck
/////////////////////////////////////////////////////////////////////////////////////

RECT XSurface::getClippingRect(void)
{
	return m_rectClippingRect;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSurface::getBitmapSize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Groesse einer Bitmap zurueck
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSurface::getBitmapSize(LPSTR	lpstrFilename,
								LPLONG	plWidth,
								LPLONG	plHeight)
{
	HBITMAP	hBitmap;
	BITMAP	bitmap;
	int		nResult;

	// Bitmap laden
	hBitmap = (HBITMAP) LoadImage(0,lpstrFilename,IMAGE_BITMAP,0,0,LR_LOADFROMFILE|LR_CREATEDIBSECTION);

	if (hBitmap==NULL)
	{ // Datei nicht gefunden
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::getBitmapSize > (FEHLER!!!) Datei \"");
		XLogging::log(lpstrFilename);
		XLogging::log("\" nicht gefunden\n");
		#endif

		return DDERR_NOTFOUND;
	}

	// Bitmap Ausmasse holen
	nResult	= GetObject(hBitmap,sizeof(BITMAP),&bitmap);

	if (nResult==0)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSurface::getBitmapSize > (FEHLER!!!) Konnte Gre von Bitmap \"");
		XLogging::log(lpstrFilename);
		XLogging::log("\" nicht ermitteln\n");
		#endif

		// Bitmap-Objekt loeschen
		DeleteObject(hBitmap);

		return DDERR_INVALIDOBJECT;
	}

	// ermittelte Werte setzen
	*plWidth	= bitmap.bmWidth;
	*plHeight	= bitmap.bmHeight;

	// Bitmap-Objekt loeschen
	DeleteObject(hBitmap);

	return DD_OK;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSurface.cpp
/////////////////////////////////////////////////////////////////////////////////////